#ifndef __THINKBAT_COLLECT_DATA__
#define __THINKBAT_COLLECT_DATA__

void read_battery_data();
int get_percent_remaining();
int get_time_remaining(int);
int read_from_file();
int get_ac_power();
float get_power_change(int);

#endif
